@@ -48,6 +48,7 @@ use deno_npm_installer::LifecycleScriptsConfig;
 use deno_npm_installer::graph::NpmCachingStrategy;
 use deno_path_util::resolve_url_or_path;
 use deno_resolver::factory::resolve_jsr_url;
+use deno_runtime::deno_node::ops::ipc::ChildIpcSerialization;
 use deno_runtime::deno_permissions::AllowRunDescriptor;
 use deno_runtime::deno_permissions::PathDescriptor;
 use deno_runtime::deno_permissions::PermissionsOptions;
@@ -541,6 +542,7 @@ impl CliOptions {
 
   pub fn graph_kind(&self) -> GraphKind {
     match self.sub_command() {
+      DenoSubcommand::Add(_) => GraphKind::All,
       DenoSubcommand::Cache(_) => GraphKind::All,
       DenoSubcommand::Check(_) => GraphKind::TypesOnly,
       DenoSubcommand::Install(InstallFlags::Local(_)) => GraphKind::All,
@@ -586,18 +588,31 @@ impl CliOptions {
     )
   }
 
-  pub fn node_ipc_fd(&self) -> Option<i64> {
+  pub fn node_ipc_init(
+    &self,
+  ) -> Result<Option<(i64, ChildIpcSerialization)>, AnyError> {
     let maybe_node_channel_fd = std::env::var("NODE_CHANNEL_FD").ok();
-    if let Some(node_channel_fd) = maybe_node_channel_fd {
-      // Remove so that child processes don't inherit this environment variable.
-
-      #[allow(clippy::undocumented_unsafe_blocks)]
-      unsafe {
-        std::env::remove_var("NODE_CHANNEL_FD")
-      };
-      node_channel_fd.parse::<i64>().ok()
+    let maybe_node_channel_serialization = if let Ok(serialization) =
+      std::env::var("NODE_CHANNEL_SERIALIZATION_MODE")
+    {
+      Some(serialization.parse::<ChildIpcSerialization>()?)
     } else {
       None
+    };
+    if let Some(node_channel_fd) = maybe_node_channel_fd {
+      // Remove so that child processes don't inherit this environment variables.
+      #[allow(clippy::undocumented_unsafe_blocks)]
+      unsafe {
+        std::env::remove_var("NODE_CHANNEL_FD");
+        std::env::remove_var("NODE_CHANNEL_SERIALIZATION_MODE");
+      }
+      let node_channel_fd = node_channel_fd.parse::<i64>()?;
+      Ok(Some((
+        node_channel_fd,
+        maybe_node_channel_serialization.unwrap_or(ChildIpcSerialization::Json),
+      )))
+    } else {
+      Ok(None)
     }
   }
 
@@ -642,12 +657,25 @@ impl CliOptions {
       return Ok(vec![]);
     }
 
-    let mut preload = Vec::with_capacity(self.flags.preload.len());
+    let mut modules = Vec::with_capacity(self.flags.preload.len());
     for preload_specifier in self.flags.preload.iter() {
-      preload.push(resolve_url_or_path(preload_specifier, self.initial_cwd())?);
+      modules.push(resolve_url_or_path(preload_specifier, self.initial_cwd())?);
     }
 
-    Ok(preload)
+    Ok(modules)
+  }
+
+  pub fn require_modules(&self) -> Result<Vec<ModuleSpecifier>, AnyError> {
+    if self.flags.require.is_empty() {
+      return Ok(vec![]);
+    }
+
+    let mut require = Vec::with_capacity(self.flags.require.len());
+    for require_specifier in self.flags.require.iter() {
+      require.push(resolve_url_or_path(require_specifier, self.initial_cwd())?);
+    }
+
+    Ok(require)
   }
 
   fn resolve_main_module_with_resolver_if_bare(
@@ -1155,9 +1183,11 @@ impl CliOptions {
         DenoSubcommand::Check(check_flags) => {
           Some(files_to_urls(&check_flags.files))
         }
-        DenoSubcommand::Install(InstallFlags::Global(flags)) => {
-          file_to_url(&flags.module_url).map(|url| vec![url])
-        }
+        DenoSubcommand::Install(InstallFlags::Global(flags)) => flags
+          .module_urls
+          .first()
+          .and_then(|url| file_to_url(url))
+          .map(|url| vec![url]),
         DenoSubcommand::Doc(DocFlags {
           source_files: DocSourceFileFlag::Paths(paths),
           ..
@@ -1348,6 +1378,7 @@ impl CliOptions {
   pub fn lifecycle_scripts_config(&self) -> LifecycleScriptsConfig {
     LifecycleScriptsConfig {
       allowed: self.flags.allow_scripts.clone(),
+      denied: Default::default(),
       initial_cwd: self.initial_cwd.clone(),
       root_dir: self.workspace().root_dir_path(),
       explicit_install: matches!(
@@ -1368,7 +1399,12 @@ impl CliOptions {
     if matches!(
       self.sub_command(),
       DenoSubcommand::Install(InstallFlags::Local(
-        InstallFlagsLocal::TopLevel | InstallFlagsLocal::Add(_)
+        InstallFlagsLocal::TopLevel(_)
+          | InstallFlagsLocal::Add(_)
+          | InstallFlagsLocal::Entrypoints(InstallEntrypointsFlags {
+            lockfile_only: true,
+            ..
+          })
       )) | DenoSubcommand::Add(_)
         | DenoSubcommand::Outdated(_)
     ) {
@@ -1545,7 +1581,7 @@ fn flags_to_permissions_options(
     }
   }
 
-  fn handle_deny(
+  fn handle_deny_or_ignore(
     value: Option<&Vec<String>>,
     config: Option<&PermissionConfigValue>,
     parse_config_value: &impl Fn(&str) -> String,
@@ -1605,7 +1641,7 @@ fn flags_to_permissions_options(
     }
     None => value.to_string(),
   };
-  let no_op = |value: &str| value.to_string();
+  let identity = |value: &str| value.to_string();
 
   Ok(PermissionsOptions {
     allow_env: handle_allow(
@@ -1613,24 +1649,29 @@ fn flags_to_permissions_options(
       config.and_then(|c| c.permissions.all),
       flags.allow_env.as_ref(),
       config.and_then(|c| c.permissions.env.allow.as_ref()),
-      &no_op,
+      &identity,
     ),
-    deny_env: handle_deny(
+    deny_env: handle_deny_or_ignore(
       flags.deny_env.as_ref(),
       config.and_then(|c| c.permissions.env.deny.as_ref()),
-      &no_op,
+      &identity,
+    ),
+    ignore_env: handle_deny_or_ignore(
+      flags.ignore_env.as_ref(),
+      config.and_then(|c| c.permissions.env.ignore.as_ref()),
+      &identity,
     ),
     allow_net: handle_allow(
       flags.allow_all,
       config.and_then(|c| c.permissions.all),
       flags.allow_net.as_ref(),
       config.and_then(|c| c.permissions.net.allow.as_ref()),
-      &no_op,
+      &identity,
     ),
-    deny_net: handle_deny(
+    deny_net: handle_deny_or_ignore(
       flags.deny_net.as_ref(),
       config.and_then(|c| c.permissions.net.deny.as_ref()),
-      &no_op,
+      &identity,
     ),
     allow_ffi: handle_allow(
       flags.allow_all,
@@ -1639,7 +1680,7 @@ fn flags_to_permissions_options(
       config.and_then(|c| c.permissions.ffi.allow.as_ref()),
       &make_fs_config_value_absolute,
     ),
-    deny_ffi: handle_deny(
+    deny_ffi: handle_deny_or_ignore(
       flags.deny_ffi.as_ref(),
       config.and_then(|c| c.permissions.ffi.deny.as_ref()),
       &make_fs_config_value_absolute,
@@ -1651,11 +1692,16 @@ fn flags_to_permissions_options(
       config.and_then(|c| c.permissions.read.allow.as_ref()),
       &make_fs_config_value_absolute,
     ),
-    deny_read: handle_deny(
+    deny_read: handle_deny_or_ignore(
       flags.deny_read.as_ref(),
       config.and_then(|c| c.permissions.read.deny.as_ref()),
       &make_fs_config_value_absolute,
     ),
+    ignore_read: handle_deny_or_ignore(
+      flags.ignore_read.as_ref(),
+      config.and_then(|c| c.permissions.read.ignore.as_ref()),
+      &make_fs_config_value_absolute,
+    ),
     allow_run: handle_allow(
       flags.allow_all,
       config.and_then(|c| c.permissions.all),
@@ -1663,7 +1709,7 @@ fn flags_to_permissions_options(
       config.and_then(|c| c.permissions.run.allow.as_ref()),
       &make_run_config_value_absolute,
     ),
-    deny_run: handle_deny(
+    deny_run: handle_deny_or_ignore(
       flags.deny_run.as_ref(),
       config.and_then(|c| c.permissions.run.deny.as_ref()),
       &make_run_config_value_absolute,
@@ -1673,12 +1719,12 @@ fn flags_to_permissions_options(
       config.and_then(|c| c.permissions.all),
       flags.allow_sys.as_ref(),
       config.and_then(|c| c.permissions.sys.allow.as_ref()),
-      &no_op,
+      &identity,
     ),
-    deny_sys: handle_deny(
+    deny_sys: handle_deny_or_ignore(
       flags.deny_sys.as_ref(),
       config.and_then(|c| c.permissions.sys.deny.as_ref()),
-      &no_op,
+      &identity,
     ),
     allow_write: handle_allow(
       flags.allow_all,
@@ -1687,7 +1733,7 @@ fn flags_to_permissions_options(
       config.and_then(|c| c.permissions.write.allow.as_ref()),
       &make_fs_config_value_absolute,
     ),
-    deny_write: handle_deny(
+    deny_write: handle_deny_or_ignore(
       flags.deny_write.as_ref(),
       config.and_then(|c| c.permissions.write.deny.as_ref()),
       &make_fs_config_value_absolute,
@@ -1697,12 +1743,12 @@ fn flags_to_permissions_options(
       config.and_then(|c| c.permissions.all),
       flags.allow_import.as_ref(),
       config.and_then(|c| c.permissions.import.allow.as_ref()),
-      &no_op,
+      &identity,
     ),
-    deny_import: handle_deny(
+    deny_import: handle_deny_or_ignore(
       flags.deny_import.as_ref(),
       config.and_then(|c| c.permissions.import.deny.as_ref()),
-      &no_op,
+      &identity,
     ),
     prompt: !resolve_no_prompt(flags),
   })
@@ -1710,6 +1756,7 @@ fn flags_to_permissions_options(
 
 #[cfg(test)]
 mod test {
+  use deno_config::deno_json::AllowDenyIgnorePermissionConfig;
   use deno_config::deno_json::AllowDenyPermissionConfig;
   use deno_config::deno_json::PermissionsObject;
   use pretty_assertions::assert_eq;
@@ -1797,7 +1844,7 @@ mod test {
           .unwrap(),
         permissions: PermissionsObject {
           all: None,
-          read: AllowDenyPermissionConfig {
+          read: AllowDenyIgnorePermissionConfig {
             allow: Some(PermissionConfigValue::Some(vec![
               ".".to_string(),
               "./read-allow".to_string(),
@@ -1805,6 +1852,9 @@ mod test {
             deny: Some(PermissionConfigValue::Some(vec![
               "./read-deny".to_string(),
             ])),
+            ignore: Some(PermissionConfigValue::Some(vec![
+              "./read-ignore".to_string(),
+            ])),
           },
           write: AllowDenyPermissionConfig {
             allow: Some(PermissionConfigValue::Some(vec![
@@ -1822,13 +1872,16 @@ mod test {
               "example.com".to_string(),
             ])),
           },
-          env: AllowDenyPermissionConfig {
+          env: AllowDenyIgnorePermissionConfig {
             allow: Some(PermissionConfigValue::Some(vec![
               "env-allow".to_string(),
             ])),
             deny: Some(PermissionConfigValue::Some(vec![
               "env-deny".to_string(),
             ])),
+            ignore: Some(PermissionConfigValue::Some(vec![
+              "env-ignore".to_string(),
+            ])),
           },
           net: AllowDenyPermissionConfig {
             allow: Some(PermissionConfigValue::Some(vec![
@@ -1873,6 +1926,7 @@ mod test {
         PermissionsOptions {
           allow_env: Some(vec!["env-allow".to_string()]),
           deny_env: Some(vec!["env-deny".to_string()]),
+          ignore_env: Some(vec!["env-ignore".to_string()]),
           allow_net: Some(vec!["net-allow".to_string()]),
           deny_net: Some(vec!["net-deny".to_string()]),
           allow_ffi: Some(vec![
@@ -1904,6 +1958,13 @@ mod test {
               .into_string()
               .unwrap()
           ]),
+          ignore_read: Some(vec![
+            base_dir
+              .join("read-ignore")
+              .into_os_string()
+              .into_string()
+              .unwrap()
+          ]),
           allow_run: Some(vec![
             "run-allow".to_string(),
             base_dir
@@ -1972,12 +2033,14 @@ mod test {
         PermissionsOptions {
           allow_env: Some(vec![]),
           deny_env: None,
+          ignore_env: None,
           allow_net: Some(vec![]),
           deny_net: None,
           allow_ffi: Some(vec![]),
           deny_ffi: None,
           allow_read: Some(vec!["./folder".to_string()]),
           deny_read: None,
+          ignore_read: None,
           allow_run: Some(vec![]),
           deny_run: None,
           allow_sys: Some(vec![]),
