@@ -1012,17 +1012,6 @@ fn diagnose_resolution(
                           module_name.to_string(),
                         ));
                       }
-                    } else if let Some(npm_resolver) = managed_npm_resolver {
-                      // check that a @types/node package exists in the resolver
-                      let types_node_req =
-                        PackageReq::from_str("@types/node").unwrap();
-                      if !npm_resolver.is_pkg_req_folder_cached(&types_node_req)
-                      {
-                        diagnostics.push(DenoDiagnostic::NotInstalledNpm(
-                          types_node_req,
-                          ModuleSpecifier::parse("npm:@types/node").unwrap(),
-                        ));
-                      }
                     }
                   } else {
                     // When the document is not available, it means that it cannot be found
