@@ -5,12 +5,15 @@ use std::sync::Arc;
 
 use deno_core::anyhow::Context;
 use deno_core::error::AnyError;
+use deno_core::futures;
 use deno_core::futures::FutureExt;
-use deno_core::futures::future::join_all;
+use deno_core::futures::StreamExt;
 use deno_core::serde_json;
 use deno_npm::resolution::NpmResolutionSnapshot;
 use deno_resolver::npmrc::npm_registry_url;
 use eszip::v2::Url;
+use http::header::HeaderName;
+use http::header::HeaderValue;
 use serde::Deserialize;
 use serde::Serialize;
 
@@ -40,14 +43,26 @@ pub async fn audit(
     .get_or_create()
     .context("Failed to create HTTP client")?;
 
-  // socket_dev::call_firewall_api(
-  //   &snapshot,
-  //   http_provider.get_or_create().unwrap(),
-  // )
-  // .await?;
+  let use_socket = audit_flags.socket;
 
-  npm::call_audits_api(audit_flags, npm_url, workspace, &snapshot, http_client)
-    .await
+  let r = npm::call_audits_api(
+    audit_flags,
+    npm_url,
+    workspace,
+    &snapshot,
+    http_client,
+  )
+  .await?;
+
+  if use_socket {
+    socket_dev::call_firewall_api(
+      &snapshot,
+      http_provider.get_or_create().unwrap(),
+    )
+    .await?;
+  }
+
+  Ok(r)
 }
 
 mod npm {
@@ -435,8 +450,8 @@ mod npm {
   #[derive(Debug, Deserialize)]
   pub struct AuditActionResolve {
     pub id: i32,
+    pub path: Option<String>,
     // TODO(bartlomieju): currently not used, commented out so it's not flagged by clippy
-    // pub path: String,
     // pub dev: bool,
     // pub optional: bool,
     // pub bundled: bool,
@@ -448,7 +463,7 @@ mod npm {
     pub is_major: bool,
     pub action: String,
     pub resolves: Vec<AuditActionResolve>,
-    pub module: String,
+    pub module: Option<String>,
     pub target: Option<String>,
   }
 
@@ -476,30 +491,41 @@ mod npm {
 
   impl AuditAdvisory {
     fn find_actions(&self, actions: &[AuditAction]) -> Vec<String> {
-      let mut acts = vec![];
+      let mut acts = Vec::new();
 
       for action in actions {
-        if action
-          .resolves
-          .iter()
-          .any(|action_resolve| action_resolve.id == self.id)
-        {
-          acts.push(format!(
-            "{} {}{}{}",
-            action.action,
-            action.module,
-            if let Some(target) = &action.target {
-              &format!("@{}", target)
-            } else {
-              ""
-            },
-            if action.is_major {
-              " (major upgrade)"
-            } else {
-              ""
-            }
-          ))
+        if !action.resolves.iter().any(|r| r.id == self.id) {
+          continue;
         }
+
+        let module = action
+          .module
+          .as_deref()
+          .map(str::to_owned)
+          .or_else(|| {
+            // Fallback to infer from dependency path
+            action.resolves.first().and_then(|r| {
+              r.path
+                .as_deref()
+                .and_then(|p| p.split('>').next_back())
+                .map(|s| s.trim().to_string())
+            })
+          })
+          .unwrap_or_else(|| "<unknown>".to_string());
+
+        let target = action
+          .target
+          .as_deref()
+          .map(|t| format!("@{}", t))
+          .unwrap_or_default();
+
+        let major = if action.is_major {
+          " (major upgrade)"
+        } else {
+          ""
+        };
+
+        acts.push(format!("{} {}{}{}", action.action, module, target, major));
       }
 
       acts
@@ -555,15 +581,77 @@ mod socket_dev {
       })
       .collect::<Vec<_>>();
 
-    // eprintln!("purls {:#?}", purls);
+    let api_key = std::env::var("SOCKET_API_KEY").ok();
+
+    let mut purl_responses = if let Some(api_key) = api_key {
+      call_authenticated_api(&client, &purls, &api_key).await?
+    } else {
+      call_unauthenticated_api(&client, &purls).await?
+    };
+
+    purl_responses.sort_by_cached_key(|r| r.name.to_string());
+
+    print_firewall_report(&purl_responses);
+
+    Ok(())
+  }
+
+  async fn call_authenticated_api(
+    client: &HttpClient,
+    purls: &[String],
+    api_key: &str,
+  ) -> Result<Vec<FirewallResponse>, AnyError> {
+    let socket_dev_url =
+      std::env::var("SOCKET_DEV_URL").ok().unwrap_or_else(|| {
+        "https://api.socket.dev/v0/purl?actions=error,warn".to_string()
+      });
+    let url = Url::parse(&socket_dev_url).unwrap();
+
+    let body = serde_json::json!({
+      "components": purls.iter().map(|purl| {
+        serde_json::json!({ "purl": purl })
+      }).collect::<Vec<_>>()
+    });
+
+    let auth_value = HeaderValue::from_str(&format!("Bearer {}", api_key))
+      .context("Failed to create Authorization header")?;
+
+    let request = client
+      .post_json(url, &body)?
+      .header(HeaderName::from_static("authorization"), auth_value);
+
+    let response = request.send().boxed_local().await?;
+    let text = http_util::body_to_string(response).await?;
+
+    // Response is nJSON
+    let responses = text
+      .lines()
+      .filter(|line| !line.trim().is_empty())
+      .map(|line| {
+        serde_json::from_str::<FirewallResponse>(line)
+          .context("Failed to parse Socket.dev response")
+      })
+      .collect::<Result<Vec<_>, _>>()?;
+
+    Ok(responses)
+  }
+
+  async fn call_unauthenticated_api(
+    client: &HttpClient,
+    purls: &[String],
+  ) -> Result<Vec<FirewallResponse>, AnyError> {
+    let socket_dev_url = std::env::var("SOCKET_DEV_URL")
+      .ok()
+      .unwrap_or_else(|| "https://firewall-api.socket.dev/".to_string());
 
     let futures = purls
-      .into_iter()
+      .iter()
       .map(|purl| {
         let url = Url::parse(&format!(
-          "https://firewall-api.socket.dev/purl/{}",
+          "{}purl/{}",
+          socket_dev_url,
           percent_encoding::utf8_percent_encode(
-            &purl,
+            purl,
             percent_encoding::NON_ALPHANUMERIC
           )
         ))
@@ -572,51 +660,189 @@ mod socket_dev {
       })
       .collect::<Vec<_>>();
 
-    // TODO(bartlomieju): run at most 20 requests at the same time, waiting on socket.dev
-    // to provide a batch API
-    let purl_results = join_all(futures).await;
+    let purl_results = futures::stream::iter(futures)
+      .buffer_unordered(20)
+      .collect::<Vec<_>>()
+      .await;
 
-    let mut purl_responses = purl_results
+    let responses = purl_results
       .into_iter()
       .filter_map(|result| match result {
         Ok(a) => Some(a),
         Err(err) => {
-          log::error!("Failed to get result {:?}", err);
+          log::error!("Failed to get PURL result {:?}", err);
           None
         }
       })
-      .map(|json_response| {
-        let response: FirewallResponse =
-          serde_json::from_str(&json_response).unwrap();
-        response
+      .filter_map(|json_response| {
+        match serde_json::from_str::<FirewallResponse>(&json_response) {
+          Ok(response) => Some(response),
+          Err(err) => {
+            log::error!("Failed deserializing socket.dev response {:?}", err);
+            None
+          }
+        }
       })
       .collect::<Vec<_>>();
-    purl_responses.sort_by_cached_key(|r| r.name.to_string());
 
+    Ok(responses)
+  }
+
+  fn print_firewall_report(responses: &[FirewallResponse]) {
     let stdout = &mut std::io::stdout();
 
-    for response in purl_responses {
-      if let Some(score) = response.score
-        && score.overall <= 0.2
-      {
-        _ = writeln!(
-          stdout,
-          "{}@{} Low score - {}",
-          response.name, response.version, score.overall
-        );
-      }
-      if !response.alerts.is_empty() {
-        for alert in response.alerts.iter() {
-          _ = writeln!(
-            stdout,
-            "{}@{} Alert - {} - {}",
-            response.name, response.version, alert.severity, alert.category
-          );
-        }
-      }
+    let responses_with_alerts = responses
+      .iter()
+      .filter(|r| !r.alerts.is_empty())
+      .collect::<Vec<_>>();
+
+    if responses_with_alerts.is_empty() {
+      return;
     }
 
-    Ok(())
+    _ = writeln!(stdout);
+    _ = writeln!(stdout, "{}", colors::bold("Socket.dev firewall report"));
+    _ = writeln!(stdout);
+
+    // Count total alerts by severity
+    let mut total_critical = 0;
+    let mut total_high = 0;
+    let mut total_medium = 0;
+    let mut total_low = 0;
+    let mut packages_with_issues = 0;
+
+    for response in responses_with_alerts {
+      packages_with_issues += 1;
+
+      _ = writeln!(stdout, "╭ pkg:npm/{}@{}", response.name, response.version);
+
+      if let Some(score) = &response.score {
+        _ = writeln!(
+          stdout,
+          "│ {:<20} {:>3}",
+          colors::gray("Supply Chain Risk:"),
+          format_score(score.supply_chain)
+        );
+        _ = writeln!(
+          stdout,
+          "│ {:<20} {:>3}",
+          colors::gray("Maintenance:"),
+          format_score(score.maintenance)
+        );
+        _ = writeln!(
+          stdout,
+          "│ {:<20} {:>3}",
+          colors::gray("Quality:"),
+          format_score(score.quality)
+        );
+        _ = writeln!(
+          stdout,
+          "│ {:<20} {:>3}",
+          colors::gray("Vulnerabilities:"),
+          format_score(score.vulnerability)
+        );
+        _ = writeln!(
+          stdout,
+          "│ {:<20} {:>3}",
+          colors::gray("License:"),
+          format_score(score.license)
+        );
+      }
+
+      // critical and high are counted as one for display.
+      let mut critical_count = 0;
+      let mut medium_count = 0;
+      let mut low_count = 0;
+
+      for alert in &response.alerts {
+        match alert.severity.as_str() {
+          "critical" => {
+            total_critical += 1;
+            critical_count += 1;
+          }
+          "high" => {
+            total_high += 1;
+            critical_count += 1;
+          }
+          "medium" => {
+            total_medium += 1;
+            medium_count += 1;
+          }
+          "low" => {
+            total_low += 1;
+            low_count += 1;
+          }
+          _ => {}
+        }
+      }
+
+      if !response.alerts.is_empty() {
+        let alerts_str = response
+          .alerts
+          .iter()
+          .map(|alert| {
+            let severity_bracket = match alert.severity.as_str() {
+              "critical" => colors::red("critical").to_string(),
+              "high" => colors::red("high").to_string(),
+              "medium" => colors::yellow("medium").to_string(),
+              "low" => "low".to_string(),
+              _ => alert.severity.clone(),
+            };
+            format!("[{}] {}", severity_bracket, alert.r#type)
+          })
+          .collect::<Vec<_>>()
+          .join(", ");
+
+        let label = format!(
+          "Alerts ({}/{}/{}):",
+          critical_count, medium_count, low_count
+        );
+        _ = writeln!(stdout, "╰ {:<20} {}", colors::gray(&label), alerts_str);
+      } else {
+        _ = writeln!(stdout, "╰");
+      }
+      _ = writeln!(stdout);
+    }
+
+    let total_alerts = total_critical + total_high + total_medium + total_low;
+
+    if total_alerts == 0 && packages_with_issues == 0 {
+      _ = writeln!(stdout, "No security alerts found from Socket.dev");
+      return;
+    }
+
+    if total_alerts > 0 {
+      _ = writeln!(
+        stdout,
+        "Found {} alerts across {} packages",
+        colors::red(total_alerts),
+        colors::bold(packages_with_issues)
+      );
+      _ = writeln!(
+        stdout,
+        "Severity: {} {}, {} {}, {} {}, {} {}",
+        colors::bold(total_low),
+        colors::bold("low"),
+        colors::yellow(total_medium),
+        colors::yellow("medium"),
+        colors::red(total_high),
+        colors::red("high"),
+        colors::red(total_critical),
+        colors::red("critical"),
+      );
+    }
+  }
+
+  fn format_score(score: f64) -> String {
+    let percentage = (score * 100.0) as i32;
+    let colored = if percentage >= 80 {
+      colors::green(percentage)
+    } else if percentage >= 60 {
+      colors::yellow(percentage)
+    } else {
+      colors::red(percentage)
+    };
+    format!("{}", colored)
   }
 
   #[derive(Debug, Deserialize)]
