@@ -16,6 +16,7 @@ use deno_ast::SourceRangedForSpanned;
 use deno_ast::SourceTextInfo;
 use deno_ast::diagnostics::Diagnostic;
 use deno_ast::swc::ast as swc_ast;
+use deno_ast::swc::atoms::Wtf8Atom;
 use deno_ast::swc::common::comments::CommentKind;
 use deno_ast::swc::ecma_visit::Visit;
 use deno_ast::swc::ecma_visit::VisitWith;
@@ -868,17 +869,18 @@ impl ReplSession {
       .imports
       .iter()
       .flat_map(|i| {
+        let specifier = i.to_string_lossy();
         self
           .resolver
           .resolve(
-            i,
+            &specifier,
             &self.referrer,
             deno_graph::Position::zeroed(),
             ResolutionMode::Import,
             NodeResolutionKind::Execution,
           )
           .ok()
-          .or_else(|| ModuleSpecifier::parse(i).ok())
+          .or_else(|| ModuleSpecifier::parse(&specifier).ok())
       })
       .collect::<Vec<_>>();
 
@@ -887,17 +889,10 @@ impl ReplSession {
       .flat_map(|url| NpmPackageReqReference::from_specifier(url).ok())
       .map(|r| r.into_inner().req)
       .collect::<Vec<_>>();
-    let has_node_specifier =
-      resolved_imports.iter().any(|url| url.scheme() == "node");
-    if !npm_imports.is_empty() || has_node_specifier {
+    if !npm_imports.is_empty() {
       npm_installer
         .add_and_cache_package_reqs(&npm_imports)
         .await?;
-
-      // prevent messages in the repl about @types/node not being cached
-      if has_node_specifier {
-        npm_installer.inject_synthetic_types_node_package().await?;
-      }
     }
     Ok(())
   }
@@ -935,7 +930,7 @@ impl ReplSession {
 /// Walk an AST and get all import specifiers for analysis if any of them is
 /// an npm specifier.
 struct ImportCollector {
-  pub imports: Vec<String>,
+  pub imports: Vec<Wtf8Atom>,
 }
 
 impl ImportCollector {
@@ -955,7 +950,7 @@ impl Visit for ImportCollector {
     if !call_expr.args.is_empty() {
       let arg = &call_expr.args[0];
       if let swc_ast::Expr::Lit(swc_ast::Lit::Str(str_lit)) = &*arg.expr {
-        self.imports.push(str_lit.value.to_string());
+        self.imports.push(str_lit.value.clone());
       }
     }
   }
@@ -969,14 +964,14 @@ impl Visit for ImportCollector {
           return;
         }
 
-        self.imports.push(import_decl.src.value.to_string());
+        self.imports.push(import_decl.src.value.clone());
       }
       ModuleDecl::ExportAll(export_all) => {
-        self.imports.push(export_all.src.value.to_string());
+        self.imports.push(export_all.src.value.clone());
       }
       ModuleDecl::ExportNamed(export_named) => {
         if let Some(src) = &export_named.src {
-          self.imports.push(src.value.to_string());
+          self.imports.push(src.value.clone());
         }
       }
       _ => {}
