@@ -23,9 +23,9 @@ use deno_graph::ModuleGraph;
 use deno_lib::util::hash::FastInsecureHasher;
 use deno_lib::worker::create_isolate_create_params;
 use deno_path_util::resolve_url_or_path;
+use deno_resolver::deno_json::JsxImportSourceConfigResolver;
 use node_resolver::ResolutionMode;
 
-use super::LAZILY_LOADED_STATIC_ASSETS;
 use super::ResolveArgs;
 use super::ResolveError;
 use crate::args::TypeCheckMode;
@@ -53,25 +53,14 @@ fn op_remap_specifier(
 #[op2]
 #[serde]
 fn op_libs() -> Vec<String> {
-  let mut out = Vec::with_capacity(LAZILY_LOADED_STATIC_ASSETS.len());
-  for (key, value) in LAZILY_LOADED_STATIC_ASSETS.iter() {
-    if !value.is_lib {
-      continue;
-    }
-    let lib = key
-      .replace("lib.", "")
-      .replace(".d.ts", "")
-      .replace("deno_", "deno.");
-    out.push(lib);
-  }
-  out
+  crate::tsc::lib_names()
 }
 
 #[op2]
 #[serde]
 fn op_resolve(
   state: &mut OpState,
-  #[string] base: String,
+  #[string] base: &str,
   #[serde] specifiers: Vec<(bool, String)>,
 ) -> Result<Vec<(String, Option<&'static str>)>, ResolveError> {
   op_resolve_inner(state, ResolveArgs { base, specifiers })
@@ -107,17 +96,17 @@ fn op_tsc_constants() -> TscConstants {
 #[inline]
 fn op_resolve_inner(
   state: &mut OpState,
-  args: ResolveArgs,
+  args: ResolveArgs<'_>,
 ) -> Result<Vec<(String, Option<&'static str>)>, ResolveError> {
   let state = state.borrow_mut::<State>();
   let mut resolved: Vec<(String, Option<&'static str>)> =
     Vec::with_capacity(args.specifiers.len());
   let referrer = if let Some(remapped_specifier) =
-    state.maybe_remapped_specifier(&args.base)
+    state.maybe_remapped_specifier(args.base)
   {
     remapped_specifier.clone()
   } else {
-    resolve_url_or_path(&args.base, &state.current_dir)?
+    resolve_url_or_path(args.base, &state.current_dir)?
   };
   let referrer_module = state.graph.get(&referrer);
   for (is_cjs, specifier) in args.specifiers {
@@ -140,6 +129,27 @@ fn op_resolve_inner(
   Ok(resolved)
 }
 
+#[op2]
+#[string]
+fn op_resolve_jsx_import_source(
+  state: &mut OpState,
+  #[string] referrer: &str,
+) -> Option<String> {
+  let state = state.borrow::<State>();
+  let referrer = if let Some(remapped_specifier) =
+    state.maybe_remapped_specifier(referrer)
+  {
+    Cow::Borrowed(remapped_specifier)
+  } else {
+    Cow::Owned(resolve_url_or_path(referrer, &state.current_dir).ok()?)
+  };
+  state
+    .jsx_import_source_config_resolver
+    .for_specifier(&referrer)?
+    .specifier()
+    .map(|s| s.to_string())
+}
+
 deno_core::extension!(deno_cli_tsc,
   ops = [
     op_create_hash,
@@ -148,6 +158,7 @@ deno_core::extension!(deno_cli_tsc,
     op_load,
     op_remap_specifier,
     op_resolve,
+    op_resolve_jsx_import_source,
     op_tsc_constants,
     op_respond,
     op_libs,
@@ -160,6 +171,7 @@ deno_core::extension!(deno_cli_tsc,
   state = |state, options| {
     state.put(State::new(
       options.request.graph,
+      options.request.jsx_import_source_config_resolver,
       options.request.hash_data,
       options.request.maybe_npm,
       options.request.maybe_tsbuildinfo,
@@ -243,11 +255,13 @@ fn op_is_node_file(state: &mut OpState, #[string] path: &str) -> bool {
   let state = state.borrow::<State>();
   ModuleSpecifier::parse(path)
     .ok()
-    .and_then(|specifier| {
+    .map(|specifier| {
       state
         .maybe_npm
         .as_ref()
         .map(|n| n.node_resolver.in_npm_package(&specifier))
+        .unwrap_or(false)
+        || specifier.as_str().starts_with("asset:///node/")
     })
     .unwrap_or(false)
 }
@@ -452,6 +466,7 @@ impl deno_core::ExtCodeCache for TscExtCodeCache {
 struct State {
   hash_data: u64,
   graph: Arc<ModuleGraph>,
+  jsx_import_source_config_resolver: Arc<JsxImportSourceConfigResolver>,
   maybe_tsbuildinfo: Option<String>,
   maybe_response: Option<RespondArgs>,
   maybe_npm: Option<RequestNpmState>,
@@ -467,6 +482,7 @@ impl Default for State {
     Self {
       hash_data: Default::default(),
       graph: Arc::new(ModuleGraph::new(GraphKind::All)),
+      jsx_import_source_config_resolver: Default::default(),
       maybe_tsbuildinfo: Default::default(),
       maybe_response: Default::default(),
       maybe_npm: Default::default(),
@@ -478,8 +494,10 @@ impl Default for State {
 }
 
 impl State {
+  #[allow(clippy::too_many_arguments)]
   pub fn new(
     graph: Arc<ModuleGraph>,
+    jsx_import_source_config_resolver: Arc<JsxImportSourceConfigResolver>,
     hash_data: u64,
     maybe_npm: Option<RequestNpmState>,
     maybe_tsbuildinfo: Option<String>,
@@ -490,6 +508,7 @@ impl State {
     State {
       hash_data,
       graph,
+      jsx_import_source_config_resolver,
       maybe_npm,
       maybe_tsbuildinfo,
       maybe_response: None,
@@ -582,6 +601,7 @@ mod tests {
       .await;
     let state = State::new(
       Arc::new(graph),
+      Default::default(),
       hash_data,
       None,
       maybe_tsbuildinfo,
@@ -601,6 +621,7 @@ mod tests {
   ) -> Result<Response, ExecError> {
     test_exec_with_cache(specifier, None).await
   }
+
   async fn test_exec_with_cache(
     specifier: &ModuleSpecifier,
     code_cache: Option<Arc<dyn deno_runtime::code_cache::CodeCache>>,
@@ -637,6 +658,7 @@ mod tests {
       config,
       debug: false,
       graph: Arc::new(graph),
+      jsx_import_source_config_resolver: Default::default(),
       hash_data,
       maybe_npm: None,
       maybe_tsbuildinfo: None,
@@ -763,7 +785,7 @@ mod tests {
     let actual = op_resolve_inner(
       &mut state,
       ResolveArgs {
-        base: "https://deno.land/x/a.ts".to_string(),
+        base: "https://deno.land/x/a.ts",
         specifiers: vec![(false, "./b.ts".to_string())],
       },
     )
@@ -785,7 +807,7 @@ mod tests {
     let actual = op_resolve_inner(
       &mut state,
       ResolveArgs {
-        base: "https://deno.land/x/a.ts".to_string(),
+        base: "https://deno.land/x/a.ts",
         specifiers: vec![(false, "./bad.ts".to_string())],
       },
     )
