@@ -352,7 +352,9 @@ impl Visit for ExportCollector {
             self.named_exports.insert(ident.sym.clone());
           }
           ast::TsModuleName::Str(s) => {
-            self.named_exports.insert(s.value.clone());
+            self
+              .named_exports
+              .insert(s.value.to_atom_lossy().into_owned());
           }
         }
       }
@@ -403,7 +405,7 @@ impl Visit for ExportCollector {
     fn get_atom(export_name: &ast::ModuleExportName) -> Atom {
       match export_name {
         ast::ModuleExportName::Ident(ident) => ident.sym.clone(),
-        ast::ModuleExportName::Str(s) => s.value.clone(),
+        ast::ModuleExportName::Str(s) => s.value.to_atom_lossy().into_owned(),
       }
     }
 
@@ -798,7 +800,7 @@ fn wrap_in_deno_test(stmts: Vec<ast::Stmt>, test_name: Atom) -> ast::Stmt {
           spread: None,
           expr: Box::new(ast::Expr::Lit(ast::Lit::Str(ast::Str {
             span: DUMMY_SP,
-            value: test_name,
+            value: test_name.into(),
             raw: None,
           }))),
         },
