@@ -942,19 +942,16 @@ impl Resource for HttpStreamWriteResource {
 }
 
 /// The read half of an HTTP stream.
+#[derive(Default)]
 pub enum HttpRequestReader {
   Headers(Request<Body>),
   Body(HeaderMap<HeaderValue>, Peekable<Body>),
+  #[default]
   Closed,
 }
 
-impl Default for HttpRequestReader {
-  fn default() -> Self {
-    Self::Closed
-  }
-}
-
 /// The write half of an HTTP stream.
+#[derive(Default)]
 enum HttpResponseWriter {
   Headers(oneshot::Sender<Response<Body>>),
   Body {
@@ -962,15 +959,10 @@ enum HttpResponseWriter {
     shutdown_handle: ShutdownHandle,
   },
   BodyUncompressed(BodyUncompressedSender),
+  #[default]
   Closed,
 }
 
-impl Default for HttpResponseWriter {
-  fn default() -> Self {
-    Self::Closed
-  }
-}
-
 struct BodyUncompressedSender(Option<hyper_v014::body::Sender>);
 
 impl BodyUncompressedSender {
