@@ -8,21 +8,18 @@ use deno_core::OpState;
 use deno_core::op2;
 use deno_core::url::Url;
 use deno_error::JsErrorBox;
+use deno_permissions::PermissionsContainer;
 
 use crate::ExtNodeSys;
-use crate::NodePermissions;
 use crate::NodeRequireLoaderRc;
 
 #[must_use = "the resolved return value to mitigate time-of-check to time-of-use issues"]
-fn ensure_read_permission<'a, P>(
+fn ensure_read_permission<'a>(
   state: &mut OpState,
   file_path: Cow<'a, Path>,
-) -> Result<Cow<'a, Path>, JsErrorBox>
-where
-  P: NodePermissions + 'static,
-{
+) -> Result<Cow<'a, Path>, JsErrorBox> {
   let loader = state.borrow::<NodeRequireLoaderRc>().clone();
-  let permissions = state.borrow_mut::<P>();
+  let permissions = state.borrow_mut::<PermissionsContainer>();
   loader.ensure_read_permission(permissions, file_path)
 }
 
@@ -72,10 +69,7 @@ pub enum WorkerThreadsFilenameError {
 // todo(dsherret): we should remove this and do all this work inside op_create_worker
 #[op2(stack_trace)]
 #[string]
-pub fn op_worker_threads_filename<
-  P: NodePermissions + 'static,
-  TSys: ExtNodeSys + 'static,
->(
+pub fn op_worker_threads_filename<TSys: ExtNodeSys + 'static>(
   state: &mut OpState,
   #[string] specifier: &str,
 ) -> Result<Option<String>, WorkerThreadsFilenameError> {
@@ -89,7 +83,7 @@ pub fn op_worker_threads_filename<
     if path.is_relative() && !specifier.starts_with('.') {
       return Err(WorkerThreadsFilenameError::InvalidRelativeUrl);
     }
-    let path = ensure_read_permission::<P>(state, Cow::Borrowed(path))
+    let path = ensure_read_permission(state, Cow::Borrowed(path))
       .map_err(WorkerThreadsFilenameError::Permission)?;
     let sys = state.borrow::<TSys>();
     let canonicalized_path =
@@ -100,7 +94,7 @@ pub fn op_worker_threads_filename<
   let url_path = url
     .to_file_path()
     .map_err(|_| WorkerThreadsFilenameError::UrlToPathString)?;
-  let url_path = ensure_read_permission::<P>(state, Cow::Owned(url_path))
+  let url_path = ensure_read_permission(state, Cow::Owned(url_path))
     .map_err(WorkerThreadsFilenameError::Permission)?;
   let sys = state.borrow::<TSys>();
   if !sys.fs_exists_no_err(&url_path) {
