@@ -21,6 +21,7 @@ use serde::Serialize;
 use serde::de;
 use serde::de::Unexpected;
 use serde::de::Visitor;
+use serde::ser::Error;
 use serde_json::Value;
 use serde_json::json;
 use sys_traits::FsRead;
@@ -39,6 +40,8 @@ use crate::util::is_skippable_io_error;
 mod permissions;
 mod ts;
 
+pub use permissions::AllowDenyIgnorePermissionConfig;
+pub use permissions::AllowDenyIgnorePermissionConfigValue;
 pub use permissions::AllowDenyPermissionConfig;
 pub use permissions::AllowDenyPermissionConfigValue;
 pub use permissions::PermissionConfigValue;
@@ -50,6 +53,71 @@ pub use ts::CompilerOptions;
 pub use ts::EmitConfigOptions;
 pub use ts::RawJsxCompilerOptions;
 
+#[derive(Clone, Debug, Hash, PartialEq)]
+pub enum AllowScriptsValueConfig {
+  All,
+  Limited(Vec<JsrDepPackageReq>),
+}
+
+impl Default for AllowScriptsValueConfig {
+  fn default() -> Self {
+    Self::Limited(Vec::new())
+  }
+}
+
+impl<'de> Deserialize<'de> for AllowScriptsValueConfig {
+  fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+  where
+    D: Deserializer<'de>,
+  {
+    struct ApprovedScriptsValueConfigVisitor;
+
+    impl<'de> Visitor<'de> for ApprovedScriptsValueConfigVisitor {
+      type Value = AllowScriptsValueConfig;
+
+      fn expecting(
+        &self,
+        formatter: &mut std::fmt::Formatter,
+      ) -> std::fmt::Result {
+        formatter.write_str("a boolean or an array of strings")
+      }
+
+      fn visit_bool<E>(self, v: bool) -> Result<Self::Value, E>
+      where
+        E: de::Error,
+      {
+        if v {
+          Ok(AllowScriptsValueConfig::All)
+        } else {
+          Ok(AllowScriptsValueConfig::Limited(Vec::new()))
+        }
+      }
+
+      fn visit_seq<A>(self, mut seq: A) -> Result<Self::Value, A::Error>
+      where
+        A: de::SeqAccess<'de>,
+      {
+        let mut items = Vec::new();
+        while let Some(item) = seq.next_element::<JsrDepPackageReq>()? {
+          items.push(item);
+        }
+        Ok(AllowScriptsValueConfig::Limited(items))
+      }
+    }
+
+    deserializer.deserialize_any(ApprovedScriptsValueConfigVisitor)
+  }
+}
+
+#[derive(Clone, Debug, Default, Deserialize, Hash, PartialEq)]
+#[serde(default, deny_unknown_fields)]
+pub struct AllowScriptsConfig {
+  #[serde(default)]
+  pub allow: AllowScriptsValueConfig,
+  #[serde(default)]
+  pub deny: Vec<JsrDepPackageReq>,
+}
+
 #[derive(Clone, Debug, Default, Deserialize, Hash, PartialEq)]
 #[serde(default, deny_unknown_fields)]
 pub struct LintRulesConfig {
@@ -875,6 +943,57 @@ pub struct NodeModulesDirParseError {
   pub source: serde_json::Error,
 }
 
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum NewestDependencyDate {
+  Enabled(chrono::DateTime<chrono::Utc>),
+  /// Disable using a minimum dependency date.
+  Disabled,
+}
+
+impl NewestDependencyDate {
+  pub fn into_option(self) -> Option<chrono::DateTime<chrono::Utc>> {
+    match self {
+      Self::Enabled(date_time) => Some(date_time),
+      Self::Disabled => None,
+    }
+  }
+}
+
+#[derive(Debug, Default, Clone, Deserialize)]
+#[serde(deny_unknown_fields, rename_all = "camelCase")]
+struct RawMinimumDependencyAgeConfig {
+  pub age: serde_json::Value,
+  #[serde(default)]
+  pub exclude: Vec<String>,
+}
+
+#[derive(Debug, Default, Clone)]
+pub struct MinimumDependencyAgeConfig {
+  pub age: Option<NewestDependencyDate>,
+  pub exclude: Vec<String>,
+}
+
+#[derive(Debug, Error, JsError)]
+#[class(type)]
+pub enum MinimumDependencyAgeParseError {
+  #[error("Unsupported \"minimumDependencyAge\" value.")]
+  ParseDateOrDuration(
+    #[from]
+    #[source]
+    crate::ParseDateOrDurationError,
+  ),
+  #[error(
+    "Unsupported \"minimumDependencyAge\" value. Expected a string or number."
+  )]
+  ExpectedStringOrNumber,
+  #[error(
+    "Unsupported \"minimumDependencyAge\" value. Could not convert number to i64."
+  )]
+  InvalidNumber,
+  #[error("Unsupported \"minimumDependencyAge\" object.")]
+  UnsupportedObject(#[source] serde_json::Error),
+}
+
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Serialize)]
 #[serde(rename_all = "kebab-case")]
 pub enum NodeModulesDirMode {
@@ -952,7 +1071,7 @@ impl NodeModulesDirMode {
 #[serde(deny_unknown_fields)]
 pub struct DeployConfig {
   pub org: String,
-  pub app: String,
+  pub app: Option<String>,
 }
 
 #[derive(Clone, Debug, Eq, PartialEq, Serialize, Deserialize)]
@@ -970,12 +1089,14 @@ pub struct ConfigFileJson {
   pub compile: Option<Value>,
   pub lock: Option<Value>,
   pub exclude: Option<Value>,
+  pub minimum_dependency_age: Option<Value>,
   pub node_modules_dir: Option<Value>,
   pub vendor: Option<bool>,
   pub license: Option<Value>,
   pub permissions: Option<Value>,
   pub publish: Option<Value>,
   pub deploy: Option<Value>,
+  pub allow_scripts: Option<Value>,
 
   pub name: Option<String>,
   pub version: Option<String>,
@@ -1423,6 +1544,10 @@ impl ConfigFile {
     self.json.name.is_some() && self.json.exports.is_some()
   }
 
+  pub fn should_publish(&self) -> bool {
+    !matches!(self.json.publish, Some(serde_json::Value::Bool(false)))
+  }
+
   pub fn is_workspace(&self) -> bool {
     self.json.workspace.is_some()
   }
@@ -1791,11 +1916,14 @@ impl ConfigFile {
   pub(crate) fn to_publish_config(
     &self,
   ) -> Result<PublishConfig, ToInvalidConfigError> {
-    match self.json.publish.clone() {
+    match &self.json.publish {
+      Some(serde_json::Value::Bool(_)) | None => Ok(PublishConfig {
+        files: self.to_exclude_files_config()?,
+      }),
       Some(config) => {
         let mut exclude_patterns = self.resolve_exclude_patterns()?;
         let mut serialized: SerializedPublishConfig =
-          serde_json::from_value(config).map_err(|error| {
+          serde_json::from_value(config.clone()).map_err(|error| {
             ToInvalidConfigError::Parse {
               config: "publish",
               source: error,
@@ -1811,9 +1939,6 @@ impl ConfigFile {
           }
         })
       }
-      None => Ok(PublishConfig {
-        files: self.to_exclude_files_config()?,
-      }),
     }
   }
 
@@ -1906,6 +2031,86 @@ impl ConfigFile {
     }
   }
 
+  pub fn to_allow_scripts_config(
+    &self,
+  ) -> Result<AllowScriptsConfig, ToInvalidConfigError> {
+    let config = match &self.json.allow_scripts {
+      Some(Value::Array(value)) => {
+        let items: Vec<JsrDepPackageReq> =
+          serde_json::from_value(value.clone().into()).map_err(|error| {
+            ToInvalidConfigError::Parse {
+              config: "allowScripts",
+              source: error,
+            }
+          })?;
+        AllowScriptsConfig {
+          allow: AllowScriptsValueConfig::Limited(items),
+          deny: vec![],
+        }
+      }
+      Some(Value::Object(value)) => {
+        let config: AllowScriptsConfig =
+          serde_json::from_value(value.clone().into()).map_err(|error| {
+            ToInvalidConfigError::Parse {
+              config: "allowScripts",
+              source: error,
+            }
+          })?;
+        config
+      }
+      Some(Value::Bool(value)) => {
+        if *value {
+          AllowScriptsConfig {
+            allow: AllowScriptsValueConfig::All,
+            deny: Vec::new(),
+          }
+        } else {
+          AllowScriptsConfig {
+            allow: AllowScriptsValueConfig::Limited(Vec::new()),
+            deny: Vec::new(),
+          }
+        }
+      }
+      Some(Value::Number(_) | Value::String(_)) => {
+        return Err(ToInvalidConfigError::Parse {
+          config: "allowScripts",
+          source: serde_json::Error::custom(
+            "expected string array, boolean, or object",
+          ),
+        });
+      }
+      Some(Value::Null) | None => AllowScriptsConfig {
+        allow: AllowScriptsValueConfig::Limited(Vec::new()),
+        deny: Vec::new(),
+      },
+    };
+    let ensure_reqs_no_tag = |reqs: &[JsrDepPackageReq]| {
+      // it's too much of a hassle to support tags,
+      // so error if someone uses one
+      for req in reqs {
+        if req.req.version_req.tag().is_some() {
+          return Err(ToInvalidConfigError::Parse {
+            config: "allowScripts",
+            source: serde_json::Error::custom(format!(
+              "tags are not supported in '{}'",
+              req
+            )),
+          });
+        }
+      }
+      Ok(())
+    };
+    match &config.allow {
+      AllowScriptsValueConfig::All => {}
+      AllowScriptsValueConfig::Limited(reqs) => {
+        ensure_reqs_no_tag(reqs)?;
+      }
+    }
+    ensure_reqs_no_tag(&config.deny)?;
+
+    Ok(config)
+  }
+
   pub fn to_deploy_config(
     &self,
   ) -> Result<Option<DeployConfig>, ToInvalidConfigError> {
@@ -1922,6 +2127,68 @@ impl ConfigFile {
     }
   }
 
+  pub fn to_minimum_dependency_age_config(
+    &self,
+    sys: &impl sys_traits::SystemTimeNow,
+  ) -> Result<MinimumDependencyAgeConfig, MinimumDependencyAgeParseError> {
+    fn parse_number(
+      minutes: &serde_json::Number,
+      sys: &impl sys_traits::SystemTimeNow,
+    ) -> Result<chrono::DateTime<chrono::Utc>, MinimumDependencyAgeParseError>
+    {
+      match minutes.as_i64() {
+        Some(minutes) => {
+          let now = chrono::DateTime::<chrono::Utc>::from(sys.sys_time_now());
+          Ok(now - chrono::Duration::minutes(minutes))
+        }
+        None => Err(MinimumDependencyAgeParseError::InvalidNumber),
+      }
+    }
+
+    fn parse_date(
+      value: &serde_json::Value,
+      sys: &impl sys_traits::SystemTimeNow,
+    ) -> Result<Option<NewestDependencyDate>, MinimumDependencyAgeParseError>
+    {
+      match value {
+        serde_json::Value::Null => Ok(None),
+        serde_json::Value::Number(minutes) => Ok(Some(
+          NewestDependencyDate::Enabled(parse_number(minutes, sys)?),
+        )),
+        serde_json::Value::String(value) => Ok(Some(
+          crate::util::parse_minutes_duration_or_date(sys, value)?,
+        )),
+        serde_json::Value::Bool(false) => {
+          Ok(Some(NewestDependencyDate::Disabled))
+        }
+        serde_json::Value::Bool(true)
+        | serde_json::Value::Object(_)
+        | serde_json::Value::Array(_) => {
+          Err(MinimumDependencyAgeParseError::ExpectedStringOrNumber)
+        }
+      }
+    }
+
+    match &self.json.minimum_dependency_age {
+      Some(v) => match v {
+        serde_json::Value::Object(_) => {
+          let obj: RawMinimumDependencyAgeConfig =
+            serde_json::from_value(v.clone())
+              .map_err(MinimumDependencyAgeParseError::UnsupportedObject)?;
+          Ok(MinimumDependencyAgeConfig {
+            age: parse_date(&obj.age, sys)?,
+            exclude: obj.exclude,
+          })
+        }
+        _ => Ok(MinimumDependencyAgeConfig {
+          age: parse_date(v, sys)?,
+          exclude: Default::default(),
+        }),
+      },
+      None => Ok(Default::default()),
+    }
+  }
+
   pub fn resolve_tasks_config(
     &self,
   ) -> Result<IndexMap<String, TaskDefinition>, ResolveTaskConfigError> {
@@ -2954,4 +3221,95 @@ mod tests {
       );
     }
   }
+
+  #[test]
+  fn test_to_allow_scripts() {
+    fn get_result(
+      text: &str,
+    ) -> Result<AllowScriptsConfig, ToInvalidConfigError> {
+      let config_specifier = root_url().join("deno.json").unwrap();
+      let config_file = ConfigFile::new(text, config_specifier).unwrap();
+      config_file.to_allow_scripts_config()
+    }
+
+    assert_eq!(
+      get_result(r#"{}"#).unwrap(),
+      AllowScriptsConfig {
+        allow: AllowScriptsValueConfig::Limited(Vec::new()),
+        deny: vec![],
+      }
+    );
+    assert_eq!(
+      get_result(
+        r#"{
+        "allowScripts": true
+      }"#
+      )
+      .unwrap(),
+      AllowScriptsConfig {
+        allow: AllowScriptsValueConfig::All,
+        deny: vec![],
+      }
+    );
+    assert_eq!(
+      get_result(
+        r#"{
+        "allowScripts": [
+          "npm:chalk",
+          "npm:package@1",
+        ]
+      }"#
+      )
+      .unwrap(),
+      AllowScriptsConfig {
+        allow: AllowScriptsValueConfig::Limited(Vec::from([
+          JsrDepPackageReq::from_str("npm:chalk").unwrap(),
+          JsrDepPackageReq::from_str("npm:package@1").unwrap(),
+        ])),
+        deny: vec![],
+      }
+    );
+    assert_eq!(
+      get_result(
+        r#"{
+        "allowScripts": {
+          "allow": [
+            "npm:chalk",
+            "npm:package@1",
+          ],
+          "deny": [
+            "npm:example@1"
+          ]
+        }
+      }"#
+      )
+      .unwrap(),
+      AllowScriptsConfig {
+        allow: AllowScriptsValueConfig::Limited(Vec::from([
+          JsrDepPackageReq::from_str("npm:chalk").unwrap(),
+          JsrDepPackageReq::from_str("npm:package@1").unwrap(),
+        ])),
+        deny: Vec::from(
+          [JsrDepPackageReq::from_str("npm:example@1").unwrap(),]
+        ),
+      }
+    );
+
+    // tag
+    assert_contains!(
+      format!(
+        "{:#?}",
+        get_result(r#"{ "allowScripts": ["npm:chalk@next"] }"#).unwrap_err()
+      ),
+      "tags are not supported in 'npm:chalk@next'"
+    );
+    assert_contains!(
+      format!(
+        "{:#?}",
+        get_result(r#"{ "allowScripts": { "allow": ["npm:chalk@next"] } }"#)
+          .unwrap_err()
+      ),
+      "tags are not supported in 'npm:chalk@next'"
+    );
+  }
 }
