@@ -14,10 +14,14 @@ use deno_resolver::factory::WorkspaceFactory;
 use deno_resolver::factory::WorkspaceFactorySys;
 use deno_resolver::lockfile::LockfileLock;
 use deno_resolver::lockfile::LockfileNpmPackageInfoApiAdapter;
+use deno_semver::jsr::JsrDepPackageReq;
+use deno_semver::package::PackageKind;
+use deno_semver::package::PackageReq;
 use futures::FutureExt;
 
 use crate::LifecycleScriptsConfig;
 use crate::NpmInstaller;
+use crate::NpmInstallerOptions;
 use crate::Reporter;
 use crate::graph::NpmCachingStrategy;
 use crate::graph::NpmDenoGraphResolver;
@@ -77,6 +81,7 @@ pub struct NpmInstallerFactory<
   resolver_factory: Arc<ResolverFactory<TSys>>,
   has_js_execution_started_flag: HasJsExecutionStartedFlagRc,
   http_client: Arc<TNpmCacheHttpClient>,
+  lifecycle_scripts_config: Deferred<Arc<LifecycleScriptsConfig>>,
   lifecycle_scripts_executor: Arc<dyn LifecycleScriptsExecutor>,
   reporter: TReporter,
   lockfile_npm_package_info_provider:
@@ -117,6 +122,7 @@ impl<
       resolver_factory,
       has_js_execution_started_flag: Default::default(),
       http_client,
+      lifecycle_scripts_config: Default::default(),
       lifecycle_scripts_executor,
       reporter,
       lockfile_npm_package_info_provider: Default::default(),
@@ -154,6 +160,62 @@ impl<
     Ok(())
   }
 
+  pub fn lifecycle_scripts_config(
+    &self,
+  ) -> Result<&Arc<LifecycleScriptsConfig>, anyhow::Error> {
+    use crate::PackagesAllowedScripts;
+
+    fn jsr_deps_to_reqs(deps: Vec<JsrDepPackageReq>) -> Vec<PackageReq> {
+      deps
+        .into_iter()
+        .filter_map(|p| {
+          if p.kind == PackageKind::Npm {
+            Some(p.req)
+          } else {
+            None
+          }
+        })
+        .collect::<Vec<_>>()
+    }
+
+    self.lifecycle_scripts_config.get_or_try_init(|| {
+      let workspace_factory = self.workspace_factory();
+      let workspace = &workspace_factory.workspace_directory()?.workspace;
+      let allow_scripts = workspace.allow_scripts()?;
+      let args = &self.options.lifecycle_scripts_config;
+      Ok(Arc::new(LifecycleScriptsConfig {
+        allowed: match &args.allowed {
+          PackagesAllowedScripts::All => PackagesAllowedScripts::All,
+          PackagesAllowedScripts::Some(package_reqs) => {
+            PackagesAllowedScripts::Some(package_reqs.clone())
+          }
+          PackagesAllowedScripts::None => match allow_scripts.allow {
+            deno_config::deno_json::AllowScriptsValueConfig::All => {
+              PackagesAllowedScripts::All
+            }
+            deno_config::deno_json::AllowScriptsValueConfig::Limited(deps) => {
+              let reqs = jsr_deps_to_reqs(deps);
+              if reqs.is_empty() {
+                PackagesAllowedScripts::None
+              } else {
+                PackagesAllowedScripts::Some(reqs)
+              }
+            }
+          },
+        },
+        denied: match &args.allowed {
+          PackagesAllowedScripts::All | PackagesAllowedScripts::Some(_) => {
+            vec![]
+          }
+          PackagesAllowedScripts::None => jsr_deps_to_reqs(allow_scripts.deny),
+        },
+        initial_cwd: args.initial_cwd.clone(),
+        root_dir: args.root_dir.clone(),
+        explicit_install: args.explicit_install,
+      }))
+    })
+  }
+
   pub fn lockfile_npm_package_info_provider(
     &self,
   ) -> Result<&LockfileNpmPackageInfoApiAdapter, anyhow::Error> {
@@ -293,6 +355,7 @@ impl<
           let workspace_npm_link_packages =
             workspace_factory.workspace_npm_link_packages()?;
           Ok(Arc::new(NpmInstaller::new(
+            self.install_reporter.clone(),
             self.lifecycle_scripts_executor.clone(),
             npm_cache.clone(),
             Arc::new(NpmInstallDepsProvider::from_workspace(
@@ -305,14 +368,15 @@ impl<
             &self.reporter,
             workspace_factory.sys().clone(),
             self.tarball_cache()?.clone(),
-            self.maybe_lockfile().await?.cloned(),
-            workspace_factory
-              .node_modules_dir_path()?
-              .map(|p| p.to_path_buf()),
-            self.options.lifecycle_scripts_config.clone(),
-            self.resolver_factory.npm_system_info().clone(),
-            workspace_npm_link_packages.clone(),
-            self.install_reporter.clone(),
+            NpmInstallerOptions {
+              maybe_lockfile: self.maybe_lockfile().await?.cloned(),
+              maybe_node_modules_path: workspace_factory
+                .node_modules_dir_path()?
+                .map(|p| p.to_path_buf()),
+              lifecycle_scripts: self.lifecycle_scripts_config()?.clone(),
+              system_info: self.resolver_factory.npm_system_info().clone(),
+              workspace_link_packages: workspace_npm_link_packages.clone(),
+            },
           )))
         }
         .boxed_local(),
