@@ -15,6 +15,7 @@ use deno_npm::resolution::PackageCacheFolderIdNotFoundError;
 use deno_npm::resolution::PackageNvNotFoundError;
 use deno_npm::resolution::PackageReqNotFoundError;
 use deno_path_util::fs::canonicalize_path_maybe_not_exists;
+use deno_semver::Version;
 use deno_semver::package::PackageNv;
 use deno_semver::package::PackageReq;
 use node_resolver::InNpmPackageChecker;
@@ -264,6 +265,19 @@ impl<TSys: ManagedNpmResolverSys> NpmPackageFolderResolver
     );
     Ok(path)
   }
+
+  fn resolve_types_package_folder(
+    &self,
+    types_package_name: &str,
+    maybe_package_version: Option<&Version>,
+    maybe_referrer: Option<&UrlOrPathRef>,
+  ) -> Option<PathBuf> {
+    self.fs_resolver.resolve_types_package_folder(
+      types_package_name,
+      maybe_package_version,
+      maybe_referrer,
+    )
+  }
 }
 
 #[derive(Debug, Clone)]
