@@ -7,8 +7,6 @@ use deno_core::extension;
 extension!(runtime,
   deps = [
     deno_webidl,
-    deno_console,
-    deno_url,
     deno_tls,
     deno_web,
     deno_fetch,
@@ -16,7 +14,6 @@ extension!(runtime,
     deno_websocket,
     deno_webstorage,
     deno_crypto,
-    deno_broadcast_channel,
     deno_node,
     deno_ffi,
     deno_net,
