@@ -1660,7 +1660,12 @@ impl Isolate {
   #[inline(always)]
   pub fn set_wasm_streaming_callback<F>(&mut self, _: F)
   where
-    F: UnitType + Fn(&mut PinScope, Local<Value>, WasmStreaming),
+    F: UnitType
+      + for<'a, 'b, 'c> Fn(
+        &'c mut PinScope<'a, 'b>,
+        Local<'a, Value>,
+        WasmStreaming,
+      ),
   {
     unsafe {
       v8__Isolate__SetWasmStreamingCallback(
@@ -2041,8 +2046,7 @@ impl Drop for OwnedIsolate {
       );
       // Safety: We need to check `this == Isolate::GetCurrent()` before calling exit()
       assert!(
-        self.cxx_isolate.as_mut() as *mut RealIsolate
-          == v8__Isolate__GetCurrent(),
+        std::ptr::eq(self.cxx_isolate.as_mut(), v8__Isolate__GetCurrent()),
         "v8::OwnedIsolate instances must be dropped in the reverse order of creation. They are entered upon creation and exited upon being dropped."
       );
       // self.dispose_scope_root();
