@@ -314,27 +314,25 @@ mod get_isolate_impls {
 
   impl GetIsolate for Local<'_, Context> {
     fn get_isolate_ptr(&self) -> *mut RealIsolate {
-      unsafe { raw::v8__Context__GetIsolate(&**self) }
+      unsafe { raw::v8__Isolate__GetCurrent() }
     }
   }
 
   impl GetIsolate for Local<'_, Message> {
     fn get_isolate_ptr(&self) -> *mut RealIsolate {
-      unsafe { raw::v8__Message__GetIsolate(&**self) }
+      unsafe { raw::v8__Isolate__GetCurrent() }
     }
   }
 
   impl GetIsolate for Local<'_, Promise> {
     fn get_isolate_ptr(&self) -> *mut RealIsolate {
-      let object: Local<Object> = (*self).into();
-      unsafe { raw::v8__Object__GetIsolate(&*object) }
+      unsafe { raw::v8__Isolate__GetCurrent() }
     }
   }
 
   impl GetIsolate for PromiseRejectMessage<'_> {
     fn get_isolate_ptr(&self) -> *mut RealIsolate {
-      let object: Local<Object> = self.get_promise().into();
-      unsafe { raw::v8__Object__GetIsolate(&*object) }
+      unsafe { raw::v8__Isolate__GetCurrent() }
     }
   }
 
@@ -932,9 +930,7 @@ impl<
     param: &'scope mut P,
     context: Local<'ct, Context>,
   ) -> P::NewScope {
-    if param.get_isolate_ptr()
-      != unsafe { raw::v8__Context__GetIsolate(&*context) }
-    {
+    if param.get_isolate_ptr() != unsafe { raw::v8__Isolate__GetCurrent() } {
       panic!(
         "{} and Context do not belong to the same Isolate",
         type_name::<P>()
